# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
"""Filter texts."""
import importlib
from pathlib import Path

PACKAGE_BASE = "qiuwenbot.filter"
NOT_LOADABLE = ("__init__.py",)

for module_file in Path(__file__).parent.glob("*.py"):
    if module_file.name not in NOT_LOADABLE:
        module_name = f".{module_file.stem}"
        importlib.import_module(module_name, PACKAGE_BASE)



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from .common import get_comment
from .filter import TextReplaceFilter, register_filter


class RemoveTagFilter(TextReplaceFilter):
    """Filter to remove a certain tag.

    Parameters
    ----------
    tag : str
        Tag name to remove.
    """

    def __init__(self, tag: str):
        self.tag = tag
        super().__init__(
            r"(?is)<{tag}(?P<params>[^>/]*)>(?P<content>.*?)</{tag}>".format(tag=tag),
            get_comment(rf"Removed {tag} tag"),
        )

    @property
    def log(self) -> str:
        return f"[[User:Njzjzbot/task3|移除{self.tag}标签]]"


@register_filter
class RemoveTimelineFilter(RemoveTagFilter):
    """Filter to remove timeline tag."""

    def __init__(self):
        super().__init__("timeline")


@register_filter
class RemoveScoreFilter(RemoveTagFilter):
    """Filter to remove score tag."""

    def __init__(self):
        super().__init__("score")


@register_filter
class RemoveMapframeFilter(RemoveTagFilter):
    """Filter to remove mapframe tag."""

    def __init__(self):
        super().__init__("mapframe")



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from .filter import TextReplaceFilter, register_filter


@register_filter
class WengeFilter(TextReplaceFilter):
    """Filter to add quote to Wen Ge."""

    def __init__(self):
        super().__init__(
            # not starts with quote
            # only replace links as there is something like 马文革
            r"([^“‘「『\[])((\[\[([^\[\]\|]+\|)?)(文革|文化大革命|四人帮)(\]\]))",
            r"\1“\2”",
        )

    @property
    def log(self) -> str:
        return "文革加引号"



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import pywikibot


def login(user: str, password: str) -> pywikibot.Site:
    """Login to qiuwen.

    Parameters
    ----------
    user
        username of the bot
    password
        password of the bot

    Returns
    -------
    pywikibot.Site
        qiuwen site
    """
    site = pywikibot.Site(
        code="zh",
        fam="qiuwen",
        user=user,
    )
    pywikibot.login.ClientLoginManager(
        site=site, user=user, password=password
    ).login_to_site()
    site.login()
    return site


def get_page(title: str, site: pywikibot.Site):
    """Get the page with the specific title.

    Parameters
    ----------
    title : str
        title of the page
    site : pywikibot.Site
        qiuwen site
    """
    return pywikibot.Page(site, title)



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re

try:
    import importlib.resources as pkg_resources
except ImportError:
    # python < 3.7
    import importlib_resources as pkg_resources

import qiuwenbot.filter

from .common import get_comment
from .filter import Filter, register_filter


@register_filter
class CleanRefsFilter(Filter):
    """Filter to clean references."""

    def __init__(self):
        """Initialize the filter."""
        self.rerefs = re.compile(r"(?is)<ref(?P<params>[^>/]*)>(?P<content>.*?)</ref>")
        self.removed_urls = pkg_resources.read_text(
            qiuwenbot.filter, "ref_blacklist.txt"
        ).splitlines()

    def filter(self, text: str) -> str:
        for match in self.rerefs.finditer(text):
            ref = match.group("content")
            removed = tuple(url in str(ref) for url in self.removed_urls)
            try:
                ii = removed.index(True)
            except ValueError:
                continue
            else:
                text = text.replace(
                    match.group(), get_comment("removed_ref site%d" % (ii,))
                )
        return text

    @property
    def log(self) -> str:
        return "[[User:Njzjzbot/task0|清理非法参考文献]]"



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from datetime import datetime, timedelta

from qiuwenbot.filter.filter import Filter, register_filter
from qiuwenbot.utils import devide_parameters, get_template_regex


class RemoveExpiredTemplateFilter(Filter):
    """Filter to remove a certain tag.

    Parameters
    ----------
    tag : str
        Tag name to remove.
    """

    def __init__(self, template: str):
        self.template = template
        self.template_regex = get_template_regex(template, end=r"\s*")

    def filter(self, text: str) -> str:
        matched = self.template_regex.finditer(text)
        for mm in matched:
            rm = False
            params = mm.group("params")
            params_dict = devide_parameters(params)
            time = params_dict.get("time")
            if time is None:
                rm = True
            else:
                try:
                    isotime = datetime.fromisoformat(time)
                except ValueError:
                    rm = True
                else:
                    # a month ago
                    if (
                        isotime.timestamp()
                        < (datetime.now() - timedelta(days=31)).timestamp()
                    ):
                        rm = True
            if rm:
                text = text.replace(mm.group(0), "")
        return text

    @property
    def log(self) -> str:
        return f"移除已过期的[[Template:{self.template}|{self.template}]]模板"


@register_filter
class RemoveExpiredCurrentFilter(RemoveExpiredTemplateFilter):
    """Filter to remove {{current}}."""

    def __init__(self):
        super().__init__("current")


@register_filter
class RemoveExpiredDeadFilter(RemoveExpiredTemplateFilter):
    """Filter to remove {{近期逝世}}."""

    def __init__(self):
        super().__init__("近期逝世")


@register_filter
class RemoveExpiredDeadFilter2(RemoveExpiredTemplateFilter):
    """Filter to remove {{最近逝世}}."""

    def __init__(self):
        super().__init__("最近逝世")


@register_filter
class RemoveExpiredDeadFilter3(RemoveExpiredTemplateFilter):
    """Filter to remove {{recent death}}."""

    def __init__(self):
        super().__init__("recent death")



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import os

from pywikibot import family


class QiuwenFamily(family.Family):
    """Qiuwen faimily."""

    name = "qiuwen"
    langs = {"zh": os.environ.get("QIUWEN_DOMAIN", "www.qiuwenbaike.cn")}

    def scriptpath(self, code):
        return ""

    def protocol(self, code):
        return "HTTPS"

    def isPublic(self):
        return False


family.Family._families["qiuwen"] = QiuwenFamily()



import textwrap

from qiuwenbot.utils import devide_parameters, get_cat_regex, get_template_regex


def test_category():
    text = textwrap.dedent(
        """\
        [[Category:abc]]
        [[分類:def]]
        [[分类:ghi]]
        """
    )
    p = get_cat_regex()
    m = p.finditer(text)
    assert [g.group("name") for g in m] == ["abc", "def", "ghi"]


def test_template():
    text = textwrap.dedent(
        """\
        {{abc|def|2=ghi}}
        {{def}}
        """
    )
    p = get_template_regex()
    m = p.finditer(text)
    assert [g.group("name") for g in m] == ["abc", "def"]
    m = p.finditer(text)
    assert [g.group("params") for g in m] == ["def|2=ghi", None]


def test_split_params():
    p = devide_parameters("def|2=ghi{{ssd|1=fd}}")
    assert p == {"1": "def", "2": "ghi"}



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from datetime import datetime, timedelta
from textwrap import dedent

from qiuwenbot.filter.filter import FilterChain, default_filters


def test_fileter():
    text = dedent(
        r"""\
        民国101年，日治時期“[[文化大革命]]”（[[文化大革命|文革]]）[[四人帮]]倒台后，
        [[蔡英文]][[中华民国总统|总统]]死了<ref>{{cite web|url=https://bbc.com/zhongwen/cywsl}}</ref>。
        <timeline>123</timeline>
        <score>456</score>
        <mapframe>789</mapframe>
        [[香港主權移交]]了！
        北京当局
        中华民国（台湾）
        中华民国台北市
        """
    )
    expected_text = dedent(
        r"""\
        2012年，日占時期“[[文化大革命]]”（“[[文化大革命|文革]]”）“[[四人帮]]”倒台后，
        [[蔡英文]]死了<!-- removed_ref site5 by njzjz/qiuwenbot -->。
        <!-- Removed timeline tag by njzjz/qiuwenbot -->
        <!-- Removed score tag by njzjz/qiuwenbot -->
        <!-- Removed mapframe tag by njzjz/qiuwenbot -->
        [[香港回归]]了！
        中国政府
        中国台湾
        中国台湾台北市
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_roc_flag():
    text = dedent(
        r"""\
        {{ROC}}
        [[Category:1992年台灣]]
        """
    )
    expected_text = dedent(
        r"""\
        {{CHN}}<!-- replaced_flag 0 by njzjz/qiuwenbot -->
        [[Category:1992年台灣]]
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_roc_flag2():
    text = dedent(
        r"""\
        {{bd|1992年}}{{ROC}}
        """
    )
    expected_text = dedent(
        r"""\
        {{bd|1992年}}{{CHN}}<!-- replaced_flag 0 by njzjz/qiuwenbot -->
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_expired_template():
    date1 = datetime.today() - timedelta(days=60)
    date2 = datetime.today()

    text = (
        dedent(
            r"""\
        {{current|time=%s}}
        {{current|time=%s}}
        {{近期逝世|time=%s}}
        {{近期逝世|time=%s}}
        """
        )
        % (date1, date2, date1, date2)
    )
    expected_text = (
        dedent(
            r"""\
        {{current|time=%s}}
        {{近期逝世|time=%s}}
        """
        )
        % (date2, date2)
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_tw_univ():
    text = dedent(
        r"""\
        国立台湾大学
        国立阳明大学
        """
    )
    expected_text = dedent(
        r"""\
        台湾大学
        台湾阳明大学
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_historical_authority():
    text = dedent(
        r"""\
        满洲国
        伪满洲国
        汪精卫政权
        清治时期
        """
    )
    expected_text = dedent(
        r"""\
        伪满洲国
        伪满洲国
        汪伪政权
        清朝时期
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text


def test_tw_with_countries():
    text = dedent(
        r"""\
        '''123'''是台湾的。
        == 台湾 ==
        澳大利亚与台湾
        == 澳大利亚 ==
        台湾、澳大利亚
        == 香港 ==
        香港345
        [[委内瑞拉]]、[[台湾]]、[[比利時]]
        """
    )
    expected_text = dedent(
        r"""\
        '''123'''是台湾的。
        == 中国台湾 ==
        澳大利亚与中国台湾
        == 澳大利亚 ==
        中国台湾、澳大利亚
        == 中国香港 ==
        香港345
        [[委内瑞拉]]、中国台湾、[[比利時]]
        """
    )
    filter = FilterChain(default_filters)

    text = filter.filter(text)
    assert text == expected_text



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re
from typing import Dict

import pywikibot
from pywikibot.textlib import ignore_case

from .bot import get_page


def archieve_page(page: pywikibot.Page, site: pywikibot.Site) -> pywikibot.Page:
    """Archieve a page.

    Parameters
    ----------
    page: pywikibot.Page
        page to archieve
    site: pywikibot.Site
        qiuwen site

    Returns
    -------
    pywikibot.Page
        page with old title
    """
    ii = 1
    oldtitle = page.title()
    while True:
        title = oldtitle + "/存档%d" % ii
        if title:
            archieve_page = get_page(title, site)
            if not archieve_page.exists():
                page.text = page.text.replace("{{Archives}}", "")
                page.save("删除archieves模板")
                page.move(title, "存档")
                oldpage = get_page(oldtitle, site)
                oldpage.text = "{{Archives}}"
                oldpage.save("加入archieves模板")
                return oldpage
        ii += 1


def get_cat_regex(name: str = r"[^\[\]]+") -> re.Pattern:
    """Get categories regex.

    Parameters
    ----------
    name : str, optional
        Name or regex of the category, by default all categories.

    Returns
    -------
    List[str]
        Categories.
    """
    namespaces = [ignore_case("Category"), ignore_case("分類"), ignore_case("分类")]
    return re.compile(
        r"\[\[ *(?P<namespace>{namespace})\s*:(?P<name>{name})\]\]".format(
            name=name, namespace="|".join(namespaces)
        )
    )


def get_template_regex(name: str = r"[^{\|#0-9][^{\|#]*?", end: str = "") -> re.Pattern:
    """Get templates regex.

    Parameters
    ----------
    name : str, optional
        Name or regex of the template, by default all templates.
    end : str, optional
        End of the template, by default "".

    Returns
    -------
    List[str]
        Templates.
    """
    return re.compile(
        rf"""
        {{{{\s*(?:msg:\s*)?
        (?P<name>({name}))\s*
        (?:\|(?P<params> [^{{]*?
                (({{{{{{[^{{}}]+?}}}}}}
                    |{{{{[^{{}}]+?}}}}
                    |{{[^{{}}]*?}}
                ) [^{{]*?
                )*?
            )?
        )?
        }}}}{end}
        """,
        re.VERBOSE | re.DOTALL,
    )


def devide_parameters(params: str) -> Dict[str, str]:
    """Devide parameters and remove subtemplate in it.

    Parameters
    ----------
    params : str
        parameter string

    Returns
    -------
    Dict[str, str]
        dict of params
    """
    if params is None:
        return {}
    # detect | in another template
    regex_template = get_template_regex()
    params = regex_template.sub("", params)
    params_dict = {}
    for ii, param in enumerate(params.split("|"), 1):
        if "=" in param:
            key, value = param.split("=", 1)
            params_dict[key.strip()] = value.strip()
        else:
            params_dict[str(ii)] = param.strip()
    return params_dict


coutries = [
    "阿富汗",
    "阿尔巴尼亚",
    "阿尔及利亚",
    "安道尔",
    "安哥拉",
    "安提瓜和巴布达",
    "阿根廷",
    "亚美尼亚",
    "澳大利亚",
    "奥地利",
    "阿塞拜疆",
    "巴哈马",
    "巴林",
    "孟加拉国",
    "巴巴多斯",
    "白俄罗斯",
    "比利时",
    "伯利兹",
    "贝宁",
    "不丹",
    "玻利维亚",
    "波斯尼亚和黑塞哥维那",
    "博茨瓦纳",
    "巴西",
    "文莱",
    "保加利亚",
    "布基纳法索",
    "布隆迪",
    "柬埔寨",
    "喀麦隆",
    "加拿大",
    "佛得角",
    "中非共和国",
    "乍得",
    "智利",
    "中国",
    "哥伦比亚",
    "科摩罗",
    "刚果（布）",
    "刚果（金）",
    "哥斯达黎加",
    "科特迪瓦",
    "克罗地亚",
    "古巴",
    "塞浦路斯",
    "捷克",
    "朝鲜",
    "丹麦",
    "吉布提",
    "多米尼加",
    "厄瓜多尔",
    "埃及",
    "萨尔瓦多",
    "赤道几内亚",
    "厄立特里亚",
    "爱沙尼亚",
    "埃塞俄比亚",
    "斐济",
    "芬兰",
    "法国",
    "加蓬",
    "冈比亚",
    "格鲁吉亚",
    "德国",
    "加纳",
    "希腊",
    "格林纳达",
    "危地马拉",
    "几内亚",
    "几内亚比绍",
    "圭亚那",
    "海地",
    "洪都拉斯",
    "匈牙利",
    "冰岛",
    "印度",
    "印度尼西亚",
    "伊朗",
    "伊拉克",
    "爱尔兰",
    "以色列",
    "意大利",
    "牙买加",
    "日本",
    "约旦",
    "哈萨克斯坦",
    "肯尼亚",
    "科索沃",
    "科威特",
    "吉尔吉斯斯坦",
    "老挝",
    "拉脱维亚",
    "黎巴嫩",
    "莱索托",
    "利比里亚",
    "利比亚",
    "立陶宛",
    "卢森堡",
    "马其顿",
    "马达加斯加",
    "马拉维",
    "马来西亚",
    "马尔代夫",
    "马里",
    "马耳他",
    "毛里塔尼亚",
    "毛里求斯",
    "墨西哥",
    "摩尔多瓦",
    "蒙古",
    "黑山",
    "摩洛哥",
    "莫桑比克",
    "缅甸",
    "纳米比亚",
    "尼泊尔",
    "荷兰",
    "新西兰",
    "尼加拉瓜",
    "尼日尔",
    "尼日利亚",
    "挪威",
    "阿曼",
    "巴基斯坦",
    "巴拿马",
    "巴布亚新几内亚",
    "巴拉圭",
    "秘鲁",
    "菲律宾",
    "波兰",
    "葡萄牙",
    "卡塔尔",
    "罗马尼亚",
    "俄罗斯",
    "卢旺达",
    "圣基茨和尼维斯",
    "圣卢西亚",
    "圣文森特和格林纳丁斯",
    "萨摩亚",
    "圣马力诺",
    "圣多美和普林西比",
    "沙特阿拉伯",
    "塞内加尔",
    "塞尔维亚",
    "塞舌尔",
    "塞拉利昂",
    "新加坡",
    "斯洛伐克",
    "斯洛文尼亚",
    "所罗门群岛",
    "索马里",
    "南非",
    "韩国",
    "南苏丹",
    "西班牙",
    "斯里兰卡",
    "苏丹",
    "苏里南",
    "斯威士兰",
    "瑞典",
    "瑞士",
    "叙利亚",
    "塔吉克斯坦",
    "坦桑尼亚",
    "泰国",
    "东帝汶",
    "多哥",
    "汤加",
    "特立尼达和多巴哥",
    "突尼斯",
    "土耳其",
    "土库曼斯坦",
    "乌干达",
    "乌克兰",
    "阿拉伯联合酋长国",
    "英国",
    "美国",
    "乌拉圭",
    "乌兹别克斯坦",
    "瓦努阿图",
    "委内瑞拉",
    "越南",
    "也门",
    "赞比亚",
    "津巴布韦",
    "中国台湾",
    "中国香港",
    "中国澳门",
]



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re

import cn2an

from .filter import Filter, register_filter


@register_filter
class ReplaceROCyear(Filter):
    """Filter to replace ROC year from a string.

    Parameters
    ----------
    pattern : str
        Pattern to replace.
    repl : str
        Replacement.
    """

    def __init__(self):
        self.re_roc_year = re.compile(
            r"(((\[\[([^\[\]]*\|)?(中(华|華))?民(国|國)\]\])|((中(华|華))?民(国|國)))(\d+|[一二三四五六七八九十]+)年)"
        )

    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """
        # group 0 is the entire string, group -1 is the year
        matched = self.re_roc_year.findall(text)
        for mm in matched:
            try:
                roc_year = int(mm[-1])
            except ValueError:
                try:
                    roc_year = cn2an.cn2an(mm[-1])
                except ValueError:
                    continue
            # 38 - 1949; prevent conversion of 民国19xx年
            if roc_year > 38 and roc_year < 1000:
                ce_year = 1911 + roc_year
                entire_year_str = mm[0]
                ce_year_str = "%d年" % ce_year
                text = text.replace(entire_year_str, ce_year_str)
                # remove duplicate
                # first remove links
                text = text.replace("[[%s]]" % ce_year_str, ce_year_str)
                text = text.replace(f"{ce_year_str}（{ce_year_str}）", ce_year_str)
                text = text.replace(f"{ce_year_str}({ce_year_str})", ce_year_str)
                text = text.replace(f"{ce_year_str}（{str(ce_year)}）", ce_year_str)
                text = text.replace(f"{ce_year_str}({str(ce_year)})", ce_year_str)
        return text

    @property
    def log(self) -> str:
        return "[[User:Njzjzbot/task4|替换非法纪年]]"



import os

import pytest
import pywikibot
from pywikibot import Page

from qiuwenbot.qwlogger import qwlogger
from qiuwenbot.task.task import Task


class UnitTestTask(Task):
    """A task for unit test.

    Parameters
    ----------
    user : str
        Username.
    password : str
        Password.
    pages : str
        Pages to operate.
    """

    def __init__(
        self,
        user: str,
        password: str,
        pages: dict,
    ):
        """Initialize."""
        super().__init__(
            user,
            password,
            pages,
            r"User:%s/unittest_log" % user,
            "单元测试",
        )

    def do(self, page: Page) -> bool:
        """Do the task."""
        page.text = "单元测试，时间戳：%s" % pywikibot.Timestamp.now().isoformat()
        try:
            page.save("njzjz/qiuwenbot: 单元测试", asynchronous=True)
        except (pywikibot.exceptions.Error,):
            qwlogger.exception("Failed to save page %s" % page.title())
            return False
        return True


@pytest.mark.skipif(
    os.environ.get("QIUWENBOT_TEST_USER") is None, reason="No test user."
)
@pytest.mark.skipif(
    os.environ.get("QIUWENBOT_TEST_PASSWORD") is None, reason="No test password."
)
def test_task():
    """Test task."""
    user = os.environ.get("QIUWENBOT_TEST_USER")
    password = os.environ.get("QIUWENBOT_TEST_PASSWORD")
    task = UnitTestTask(
        user=user,
        password=password,
        pages={
            "type": "page",
            "name": "User:%s/unittest" % user,
        },
    )
    task.submit()



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re

from zhconv import convert_for_mw

from qiuwenbot.utils import coutries

from .filter import Filter, TextReplaceFilter, register_filter


@register_filter
class TWLeaderFilter(TextReplaceFilter):
    """Filter to fix the leader name in the Taiwan area."""

    def __init__(self):
        super().__init__(
            (
                r"((\[\[)?(蒋介石|蒋中正|严家淦|蒋经国|李登辉|陈水扁|蔡英文|蔣介石|蔣中正|嚴家淦|蔣經國|李登輝|陳水扁)(\]\])?)"
                r"((\[\[([^\|\[\]]+\|)?)?(總統|总统)(\]\])?)"
            ),
            r"\1",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语1"


@register_filter
class TWJPFilter(TextReplaceFilter):
    """Filter to fix the Japanese authorities."""

    def __init__(self):
        super().__init__(
            r"日治(时期|時期)",
            r"日占\1",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语2"


@register_filter
class TWQingFilter(TextReplaceFilter):
    """Filter to fix the Qing authorities."""

    def __init__(self):
        super().__init__(
            r"清治(时期|時期)",
            r"清朝\1",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语2-2"


@register_filter
class TWUnivFilter1(TextReplaceFilter):
    """Filter to fix the name of unversities in the Taiwan area."""

    def __init__(self):
        super().__init__(
            r"(國立|国立)((臺|台)(灣|湾)((师范|師範|海洋|藝術|艺术|体育(运动)?|科技)?大(學|学)|戲曲學院|戏曲学院)|金门大学|金門大學)",
            r"\2",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语3"


@register_filter
class TWUnivFilter2(TextReplaceFilter):
    """Filter to fix the name of unversities in the Taiwan area."""

    def __init__(self):
        # only fix univ created after 1949
        super().__init__(
            r"(國立|国立)((高雄师范|高雄師範|彰化師範|彰化师范|台北艺术|臺北藝術|臺南|台南|體育|体育|阳明|陽明|阳明交通|陽明交通)大(学|學)|傳統藝術中心|传统艺术中心)",
            r"台湾\2",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语4"


@register_filter
class TWNameFilter1(TextReplaceFilter):
    """Filter to fix the name of the Taiwan area."""

    def __init__(self):
        super().__init__(
            r"((中华民国|中華民國)(\||\]\]\[\[)?(台|臺)(湾|灣)|(中华民国|中華民國)（(台|臺)(湾|灣)）)",
            r"中国台湾",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语5"


@register_filter
class TWNameFilter2(TextReplaceFilter):
    """Filter to fix the name of the Taiwan area."""

    def __init__(self):
        super().__init__(
            r"(中华民国|中華民國|\[\[中华民国\]\]|\[\[中華民國\]\])(臺北|台北|新北|桃园|桃園|台中|臺中|台南|臺南|高雄)",
            r"中国台湾\2",
        )

    @property
    def log(self) -> str:
        return "修正涉台用语6"


@register_filter
class TWWithOthersFilter1(TextReplaceFilter):
    """Filter to fix the Taiwan name when it is with other countries."""

    def __init__(self):
        countries_hant = [convert_for_mw(xx, "zh-hant") for xx in coutries]
        coutries_re = "|".join(coutries + countries_hant)

        super().__init__(
            r"((%s)(\]\])?(和|与|、|,|，|或|或者|及|以及))(台湾|台灣|臺湾|\[\[台湾\]\]|\[\[台灣\]\]|\[\[臺湾\]\])"
            % coutries_re,
            r"\1中国台湾",
        )

    @property
    def log(self) -> str:
        return "修正中国台湾与国家并列时的称呼1"


@register_filter
class TWWithOthersFilter2(TextReplaceFilter):
    """Filter to fix the Taiwan name when it is with other countries."""

    def __init__(self):
        countries_hant = [convert_for_mw(xx, "zh-hant") for xx in coutries]
        coutries_re = "|".join(coutries + countries_hant)

        super().__init__(
            r"([^国國])(台湾|台灣|臺湾|\[\[台湾\]\]|\[\[台灣\]\]|\[\[臺湾\]\])((和|与|、|,|，|或|或者|及|以及)(\[\[)?(%s))"
            % coutries_re,
            r"\1中国台湾\3",
        )

    @property
    def log(self) -> str:
        return "修正中国台湾与国家并列时的称呼2"


@register_filter
class TWWithOthersInTitleFilter(Filter):
    """Filter to fix the Taiwan name in title when it is with other countries."""

    def __init__(self):
        self.sections_re = re.compile(r"\n\s*[=]{2,5}\s*[^=]+\s*[=]{2,5}\s*\n")
        countries_hant = [convert_for_mw(xx, "zh-hant") for xx in coutries]
        self.coutries_re = re.compile(
            r"\n\s*[=]{2,5}\s*(%s)\s*[=]{2,5}\s*\n"
            % "|".join(coutries + countries_hant)
        )
        # fix hk and mc by the way
        self.tw_re = re.compile(
            r"(\n\s*[=]{2,5}\s*)(台湾|台灣|臺湾|\[\[台湾\]\]|\[\[台灣\]\]|\[\[臺湾\]\]|香港|澳門|澳门)(\s*[=]{2,5}\s*\n)"
        )

    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """
        sections = self.sections_re.findall(text)
        level_tw = set()
        level_country = set()
        for section in sections:
            if self.tw_re.search(section):
                level_tw.add(section.count("=") // 2)
            if self.coutries_re.search(section):
                level_country.add(section.count("=") // 2)
        if level_tw & level_country:
            # might be over-replacing, but is fast
            text = self.tw_re.sub(r"\1中国\2\3", text)
        return text

    @property
    def log(self) -> str:
        return "修正中国台湾与国家并列时的称呼3"



import json
from pathlib import Path

import pytest

from qiuwenbot.argparse import normalize

p_examples = Path(__file__).parent.parent / "examples"


@pytest.mark.parametrize(
    "p_example",
    [
        (p_examples / "filter_all.json"),
        (p_examples / "filter_new.json"),
        (p_examples / "check_duplicate.json"),
    ],
)
def test_examples(p_example):
    """Test examples."""
    with open(p_example) as f:
        config = json.load(f)
    config = normalize(config)



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
"""Check duplicated page with different variants
of Chinese titles, such as zh-cn and zh-hk.
"""
import pywikibot
from pywikibot import Page, Site
from zhconv import convert_for_mw

from qiuwenbot.bot import get_page
from qiuwenbot.qwlogger import qwlogger
from qiuwenbot.task.task import Task

# variants = ("zh-cn", "zh-tw", "zh-hk")
variants = ("zh-hans", "zh-hant")


def check_page(page: Page, site: Site):
    """Check if a page has duplicated variants.

    arameters
    ----------
    page: pywikibot.Page
        page to check
    site : pywikibot.Site
        qiuwen site
    user : str
        username of the bot
    """
    title = page.title()
    for variant in variants:
        title_v = convert_for_mw(title, variant)
        page_v = get_page(title_v, site)
        if page_v != page and page_v.exists():
            if not page_v.isRedirectPage():
                if page.text.startswith(
                    "<noinclude>{{delete|"
                ) or page_v.text.startswith("<noinclude>{{delete|"):
                    # has been marked to delete
                    continue
                # duplicated pages A2
                page_v.text = (
                    f"<noinclude>{{{{delete|A2|c1=[[User:Njzjzbot/task2|Njzjzbot]]发现-{{'''{title_v}'''}}-与-{{[[{title}]]}}-仅有简繁差异；请管理员复查页面历史记录，合并差异[[Category:Njzjzbot/A2]]}}}}</noinclude>\n"
                    + page_v.text
                )
            else:
                # duplicated redirects R1
                if (
                    variant in ("zh-cn", "zh-hans")
                    and page_v.getRedirectTarget() == page
                ):
                    # do not process zh-hans redirect, otherwise it is wrong
                    continue
                if (
                    convert_for_mw(title_v, "zh-cn") != convert_for_mw(title, "zh-cn")
                    and page_v.getRedirectTarget() == page
                ):
                    # technical issue
                    continue
                page_v.text = (
                    f"<noinclude>{{{{delete|R1|c1=[[User:Njzjzbot/task2|Njzjzbot]]发现-{{'''{title_v}'''}}-与-{{[[:{title}]]}}-仅有简繁差异[[Category:Njzjzbot/R1]]}}}}</noinclude>\n"
                    + page_v.text
                )
            page_v.save(
                f"[[User:Njzjzbot/task2|标记速删模板]]：[[{title_v}]]与[[{title}]]仅有简繁差异",
                asynchronous=True,
            )


class CheckDuplicatedPageTask(Task):
    """A task to check duplicated pages.

    Parameters
    ----------
    user : str
        Username.
    password : str
        Password.
    pages : str
        Pages to operate.
    """

    def __init__(
        self,
        user: str,
        password: str,
        pages: dict,
    ):
        """Initialize."""
        super().__init__(
            user,
            password,
            pages,
            r"User:%s/check_duplicated_log" % user,
            "检查重复页面",
        )

    def do(self, page: Page) -> bool:
        """Do the task."""
        if page.isRedirectPage():
            return False
        try:
            check_page(page, self.site)
        except pywikibot.exceptions.Error:
            qwlogger.exception("Failed to save page %s" % page.title())
            return False
        return True



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

import pywikibot
from pywikibot import Page

from qiuwenbot.filter.filter import FilterChain, default_filters
from qiuwenbot.qwlogger import qwlogger

from .task import Task


class FilterTask(Task):
    """A task to pass filters.

    Parameters
    ----------
    user : str
        Username.
    password : str
        Password.
    pages : str
        Pages to operate.
    """

    def __init__(
        self,
        user: str,
        password: str,
        pages: dict,
    ):
        """Initialize."""
        super().__init__(
            user,
            password,
            pages,
            r"User:%s/filter_log" % user,
            "综合治理",
        )
        self.filter = FilterChain(default_filters)

    def do(self, page: Page) -> bool:
        """Do the task."""
        text = page.text
        new_text = self.filter.filter(page.text)
        if new_text == text:
            return False
        page.text = new_text
        try:
            page.save(self.filter.log, asynchronous=True)
        except (pywikibot.exceptions.Error,):
            qwlogger.exception("Failed to save page %s" % page.title())
            return False
        return True



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re
from abc import ABCMeta, abstractmethod
from typing import List, Union


class Filter(metaclass=ABCMeta):
    """Filter texts."""

    @abstractmethod
    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """

    @property
    def log(self) -> Union[str, None]:
        """Log of the filter.

        Returns
        -------
        str
            Log of the filter.
        """


class TextReplaceFilter(Filter):
    """Filter to replace texts.

    Parameters
    ----------
    pattern : str
        Pattern to replace.
    repl : str
        Replacement.
    """

    def __init__(self, pattern: str, repl: str) -> None:
        super().__init__()
        self.prog = re.compile(pattern)
        self.repl = repl

    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """
        return self.prog.sub(self.repl, text)

    @property
    def log(self) -> str:
        """Log of the filter.

        Returns
        -------
        str
            Log of the filter.
        """
        return f"替换{self.prog.pattern}为{self.repl}"


class FilterChain(Filter):
    """Filter chain.

    Parameters
    ----------
    filters : list of Filter
        Filters to apply.
    """

    def __init__(self, filters: List[Filter]) -> None:
        super().__init__()
        self.filters = [filter() for filter in filters]
        self.active_filters = []

    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """
        active_filters = []
        for filter in self.filters:
            old_text = text
            text = filter.filter(text)
            if text != old_text:
                active_filters.append(filter)
        self.active_filters = active_filters
        return text

    @property
    def log(self) -> str:
        """Log of the filter.

        Returns
        -------
        str
            Log of the filter.
        """
        logs = []
        for filter in self.active_filters:
            if filter.log is not None:
                logs.append(filter.log)
        return "综合治理：" + "；".join(logs)


default_filters = []


def register_filter(cls: Filter) -> Filter:
    """Return a decorator to register filter.

    The filter should not have any parameters in its constructor.

    Parameters
    ----------
    cls : Filter
        Filter to register.

    Returns
    -------
    Filter
        Registered filter.

    Examples
    --------
    >>> @register_filter()
    ... class Filter1(Filter):
    ...     pass
    """
    default_filters.append(cls)
    return cls



import argparse

from qiuwenbot.entrypoints.submit import submit


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    parser_submit = subparsers.add_parser("submit", help="Submit a task")
    parser_submit.add_argument("CONFIG", type=str, help="Path to the config file")
    parser_submit.set_defaults(func=submit)
    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from abc import ABCMeta, abstractmethod

from pywikibot import Page, Timestamp
from pywikibot.pagegenerators import PreloadingGenerator
from tqdm import tqdm
from tqdm.contrib.logging import logging_redirect_tqdm

from qiuwenbot.bot import get_page, login
from qiuwenbot.qwlogger import qwlogger
from qiuwenbot.utils import archieve_page


class Task(metaclass=ABCMeta):
    """A task to be done.

    Parameters
    ----------
    user : str
        Username.
    password : str
        Password.
    pages : dict
        Pages to operate.
    logging_page : str, optional
        Page to log the task, by default None
    summary : str, optional
        Summary of the task, by default emptry string
    """

    def __init__(
        self,
        user: str,
        password: str,
        pages: dict,
        logging_page: str = None,
        summary: str = "",
    ):
        """Initialize."""
        self.site = login(user, password)
        if logging_page is not None:
            self.logging_page = get_page(logging_page, self.site)
        else:
            self.logging_page = None
        if pages["type"] == "all":
            if pages.get("restart", False):
                last_item = self.logging_page.text.strip().split("\n")[-1]
                title = last_item.split("-")[0].strip()[4:-2]
                qwlogger.info("restart from %s" % title)
            else:
                title = ""
            self.pages = self.site.allpages(
                namespace=pages.get("namespace", 0), start=title
            )
        elif pages["type"] == "new":
            start = pages.get("start", None)
            if start is not None:
                start = Timestamp.fromISOformat(start)
            end = pages.get("end", None)
            if end is not None:
                end = Timestamp.fromISOformat(end)
            self.pages = (
                change[0]
                for change in self.site.newpages(
                    returndict=True,
                    namespaces=pages.get("namespace", 0),
                    start=start,
                    end=end,
                )
            )
        elif pages["type"] in ("link", "template"):
            template = get_page(pages["name"], self.site)
            self.pages = template.getReferences(namespaces=pages.get("namespace", None))
        elif pages["type"] == "page":
            self.pages = [get_page(pages["name"], self.site)]
        else:
            raise RuntimeError("Unsupported pages type")
        self.summary = summary

    @abstractmethod
    def do(self, page: Page) -> bool:
        """Do the task."""
        raise NotImplementedError

    def logging(self, title: str) -> None:
        """Log the removing operator.

        Parameters
        ----------
        title : str
            title of the modified page
        """
        if self.logging_page is not None:
            if len(self.logging_page.text.split("\n")) > 2000:
                self.logging_page = archieve_page(self.logging_page, self.site)
            self.logging_page.text += "\n# [[%s]] - ~~~~~" % title
            self.logging_page.save(self.summary, asynchronous=True)

    def submit(self):
        """Submit the task."""
        with logging_redirect_tqdm():
            n_modified = tqdm(position=1, desc="Modified pages")
            for page in tqdm(PreloadingGenerator(self.pages), desc="Scanned pages"):
                if self.do(page):
                    n_modified.update(1)
                    self.logging(page.title())



import argparse
import json
import os

from qiuwenbot.argparse import normalize
from qiuwenbot.task.duplicate import CheckDuplicatedPageTask
from qiuwenbot.task.filter import FilterTask


def submit(args: argparse.Namespace):
    """Submit a task."""
    with open(args.CONFIG) as f:
        config = json.load(f)
    config = normalize(config)
    try:
        config["password"] = os.environ["QIUWENBOT_PASSWORD"]
    except KeyError as e:
        raise KeyError("Please set environment variable QIUWENBOT_PASSWORD") from e

    if config["task"] == "filter":
        task_class = FilterTask
    elif config["task"] == "duplicate":
        task_class = CheckDuplicatedPageTask
    else:
        raise RuntimeError("Unsupported task type")
    config.pop("task")
    task = task_class(**config)
    task.submit()



# qiuwenbot, a bot to contribute to qiuwenbaike.cn
# Copyright (C) 2023  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from .filter import TextReplaceFilter, register_filter


@register_filter
class FakeWangFilter(TextReplaceFilter):
    """Filter for Fake Wang authorities."""

    def __init__(self):
        super().__init__(
            r"(汪(精(卫|衛)|兆(铭|銘))|\[\[汪(精(卫|衛)|兆(铭|銘))\]\])(政权|政權)",
            r"汪伪政权",
        )

    @property
    def log(self) -> str:
        return "修正汪伪政权名称"


@register_filter
class FakeManchukuoFilter(TextReplaceFilter):
    """Filter for Fake Manchukuo authorities."""

    def __init__(self):
        super().__init__(
            r"([^伪僞偽])(滿|满)洲(国|國)",
            r"\1伪满洲国",
        )

    @property
    def log(self) -> str:
        return "修正伪满洲国名称"



from typing import List

from dargs import Argument, Variant


def page_variant() -> Variant:
    doc_type = "Method to scan pages."
    doc_type_all = "Scan all pages in alphabetical order."
    doc_type_new = "Scan new pages."
    doc_type_link = "Scan pages that link to a page or include a template."
    doc_type_page = "Scan a single page."
    doc_link_name = "Name of the page or template."
    doc_namespace = "Namespace(s) of the pages."
    doc_start = "Start time in ISO format."
    doc_end = "End time in ISO format."
    doc_restart = "Restart from the last page in the log."

    return Variant(
        "type",
        [
            Argument(
                "all",
                dtype=dict,
                sub_fields=[
                    Argument(
                        "namespace",
                        dtype=int,
                        optional=True,
                        default=0,
                        doc=doc_namespace,
                    ),
                    Argument(
                        "restart",
                        dtype=bool,
                        optional=True,
                        default=False,
                        doc=doc_restart,
                    ),
                ],
                doc=doc_type_all,
            ),
            Argument(
                "new",
                dtype=dict,
                sub_fields=[
                    Argument(
                        "namespace",
                        dtype=[int, list],
                        optional=True,
                        default=0,
                        doc=doc_namespace,
                    ),
                    Argument(
                        "start", dtype=str, optional=True, default=None, doc=doc_start
                    ),
                    Argument(
                        "end", dtype=str, optional=True, default=None, doc=doc_end
                    ),
                ],
                doc=doc_type_new,
            ),
            Argument(
                "link",
                dtype=dict,
                sub_fields=[
                    Argument("name", dtype=str, doc=doc_link_name),
                    Argument(
                        "namespace",
                        dtype=[int, list],
                        optional=True,
                        default=None,
                        doc=doc_namespace,
                    ),
                ],
                alias=["template"],
                doc=doc_type_link,
            ),
            Argument(
                "page",
                dtype=dict,
                sub_fields=[
                    Argument("name", dtype=str, doc=doc_link_name),
                ],
                doc=doc_type_page,
            ),
        ],
        doc=doc_type,
    )


def submit_args() -> List[Argument]:
    doc_user = "Username."
    doc_pages = "Configurations of scanned pages."
    doc_task = "Task to submit."
    return [
        Argument("user", dtype=str, doc=doc_user),
        Argument("pages", dtype=dict, doc=doc_pages, sub_variants=[page_variant()]),
        Argument("task", dtype=str, doc=doc_task),
    ]


def normalize(data: dict) -> dict:
    base = Argument("base", dtype=dict, sub_fields=submit_args())
    base.check_value(data, strict=True)
    return base.normalize(data)



# qiuwenbot, a bot to contribute to qiuwen.wiki
# Copyright (C) 2022  Jinzhe Zeng
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
import re

from qiuwenbot.utils import devide_parameters, get_cat_regex, get_template_regex

from .common import get_comment
from .filter import Filter, register_filter


@register_filter
class ReplaceROCyear(Filter):
    """Filter to replace ROC flag from a string.

    Parameters
    ----------
    pattern : str
        Pattern to replace.
    repl : str
        Replacement.
    """

    def __init__(self):
        self.re_bd = get_template_regex(r"[Bb]d")
        self.re_year = re.compile(r"(?P<year>\d+)年")
        self.re_found = get_cat_regex(
            r"(?P<year>\d+)年(.*)(设立|建立|成立|创建|創建|設立|启用|啟用)(.*)"
        )
        self.re_event = get_cat_regex(r"(?P<year>\d+)年(台灣|台湾|台灣)(.*)")

        self.re_roc = re.compile(r"({{[\s]*ROC[\s]*}})")
        self.re_roc_flagicon = re.compile(r"({{[\s]*[Ff]lagicon[\s]*\|[\s]*ROC[\s]*}})")
        self.re_nationality = re.compile(
            r"\|[\s]*(國籍|国籍|[Nn]ationality)[\s]*=[\s]*({{[\s]*ROC[\s]*}})"
        )
        self.re_death_place = re.compile(
            r"\|[\s]*(逝世地點|逝世地点|[Dd]eath_place|[Pp]lace_of_death|[Rr]esting_place)[\s]*=[\s]*({{[\s]*ROC[\s]*}})"
        )
        self.re_death_place_flagicon = re.compile(
            r"\|[\s]*(逝世地點|逝世地点|[Dd]eath_place|[Pp]lace_of_death|[Rr]esting_place)[\s]*=[\s]*({{[\s]*flagicon[\s]*\|[\s]*ROC[\s]*}})"
        )

        self.comment = get_comment("replaced_flag 0")
        self.replaced_chn = r"{{CHN}}" + self.comment
        self.replaced_chn_flagicon = r"{{flagicon|CHN}}" + self.comment
        self.replaced_chn_nationality = r"|\1={{PRC-TWN}}" + self.comment
        self.replaced_chn_no_flag = r"|\1=[[中国]]" + self.comment
        self.replaced_death_place = r"|\1={{CHN}}" + self.comment
        self.replaced_death_place_flagicon = r"|\1={{flagicon|CHN}}" + self.comment

    def filter(self, text: str) -> str:
        """Filter text.

        Parameters
        ----------
        text : str
            Text to filter.

        Returns
        -------
        str
            Filtered text.
        """
        yy = []
        replace_all = False
        replace_nationality = False
        replace_death = False
        replace_nationality_no_flag = False
        m_bd = self.re_bd.search(text)
        im_found = self.re_found.finditer(text)
        im_event = self.re_event.finditer(text)

        birth = None
        death = None
        if m_bd is not None:
            params = m_bd.group("params")
            params_dict = devide_parameters(params)
            birth = params_dict.get("1")
            death = params_dict.get("3", "living")

        if birth is not None:
            m_birth = self.re_year.search(birth)
            if m_birth is not None:
                y = m_birth.group("year")
                if int(y) > 1949:
                    # born after 1949
                    replace_all = True
                    replace_nationality = True

        if death is not None:
            m_death = self.re_year.search(death)
            if m_death is not None:
                y = m_death.group("year")
                if int(y) > 1949:
                    # death after 1949
                    replace_death = True
                    replace_nationality_no_flag = True
            elif death == "living":
                # living person
                replace_nationality_no_flag = True

        for m_found in im_found:
            y = m_found.group("year")
            yy.append(int(y))

        for m_event in im_event:
            y = m_event.group("year")
            yy.append(int(y))
        if len(yy) and min(yy) > 1949:
            replace_all = True
        if replace_nationality:
            # nationality
            text = self.re_nationality.sub(self.replaced_chn_nationality, text)
        if replace_nationality_no_flag:
            # nationality
            text = self.re_nationality.sub(self.replaced_chn_no_flag, text)
        if replace_death:
            text = self.re_death_place.sub(self.replaced_death_place, text)
            text = self.re_death_place_flagicon.sub(
                self.replaced_death_place_flagicon, text
            )
        if replace_all:
            text = self.re_roc.sub(self.replaced_chn, text)
            text = self.re_roc_flagicon.sub(self.replaced_chn_flagicon, text)
        return text

    @property
    def log(self) -> str:
        return "[[User:Njzjzbot/task1|清理台当局非法旗帜]]"
